jQuery.cachedScript = function(url, options) {
  options = $.extend(options || {}, {
    dataType: "script",
    cache: true,
    url: url
  });
  return jQuery.ajax(options);
};

/**
 * Controla funções que utilizam mapas e GPS
 * @type {Object}
 */
var gps = {

  initialize: function() {
    $.cachedScript("http://maps.googleapis.com/maps/api/js?key=AIzaSyBv9ZKR7s3jRIaDyK2DG1ZT26Id2HMZcvw&sensor=true&v=3.exp&callback=gps.setMapIni").done(function(script, textStatus) {
      // console.log(textStatus);
    });
  },

  /**
   * Busca localização para carregar o mapa pela primeira vez que o aplicativo
   * é aberto
   */
  setMapIni: function() {
    navigator.geolocation.getCurrentPosition(gps.setMap, gps.onError, {
      maximumAge: 5000
    });
    user.postInfo();
  },

  /**
   * @param {position} - cordova-plugin-geolocation
   */
  setMap: function(position) {
    lLatitude = aLatitude;
    lLongitude = aLongitude;
    aLatitude = position.coords.latitude;
    aLongitude = position.coords.longitude;
    var latLong = new google.maps.LatLng(aLatitude, aLongitude);
    var mapOptions = {
      center: latLong,
      zoom: 16,
      disableDefaultUI: true,
      mapTypeId: google.maps.MapTypeId.ROADMAP
    };
    map = new google.maps.Map(document.getElementById("map"), mapOptions);

    /* RETOMA A CORRIDA QUANDO O APP É ENCERRADO. */
    if (localStorage.idRide) {
      returnCall();
    } else {
      marker_worker = new google.maps.Marker({
        position: latLong,
        map: map,
        icon: "img/marker-moto.svg"
      });
    }

    $(gps.centerLocationButton);
    // Inicializa a variável de push notifications
    pushNotification = window.plugins.pushNotification;
    // Recebe regid e executa user.ping para o servidor
    gcm.initialize();
  },

  /**
   * @param  {object} error - Objeto que contém informações do erro
   * @return {null}
   */
  onError: function(error) {
    alert('code: ' + error.code + '\n' +
      'message: ' + error.message + '\n');
  },

  /**
   * Escuta o GPS pela posição
   * @param {boolean} status - verifica se é necessário continuar buscando posição
   */
  setWatchMotion: function(info) {
    if (info === true && !watchID) {
      // console.log("setWatchMotion, watch! ("+info+")");
      watchID = navigator.geolocation.watchPosition(gps.updateCoords, app.onError, {
        enableHighAccuracy: true,
        maximumAge: 1000
      });
    } else if (info === false) {
      // console.log("setWatchMotion, stop! ("+info+")");
      navigator.geolocation.clearWatch(watchID);
      watchID = false;
    } else if ($.isNumeric(info)) {
      // console.log("setWatchMotion, wait! ("+info+")");
      var time = info;
      gps.setWatchMotion(false);
      timeout = setTimeout(function() {
        gps.setWatchMotion(true);
      }, time);
    }
  },

  /**
   * Obtém e atualiza a localização
   * @param {position} - cordova-plugin-geolocation
   * @return {null}
   */
  updateCoords: function(position) {
    // debug.msg(JSON.stringify(position));
    var longitude = position.coords.longitude;
    var latitude = position.coords.latitude;
    var latLong = new google.maps.LatLng(aLatitude, aLongitude);
    if (position.coords.accuracy < 80) {
      if (gps.getDistance(lLatitude, lLongitude, latitude, longitude) > deslocamentoMin) {
        lLatitude = aLatitude;
        lLongitude = aLongitude;
        aLatitude = latitude;
        aLongitude = longitude;
        gps.notifyMotion();
        map.setCenter(latLong);
      } else {
        aLongitude = longitude;
        aLatitude = latitude;
      }
      if (!showPanel) {
        gps.setWatchMotion(10000);
      }
    } else {
      // debug.msg("[gps.updateCoords] Low accuracy: " + position.coords.accuracy);
    }
    marker_worker.setPosition({
      lat: latitude,
      lng: longitude
    });
  },

  /**
   * Recarrega o mapa verificando se é necessário ou não incluir rotas
   * @param  {float} latitude
   * @param  {float} longitude
   * @return {null}
   */
  reloadMap: function(latitude, longitude) {
    var latLong = new google.maps.LatLng(latitude, longitude);
    var mapOptions = {
      center: latLong,
      zoom: 16,
      disableDefaultUI: true,
      mapTypeId: google.maps.MapTypeId.ROADMAP
    };
    map = new google.maps.Map(document.getElementById("map"), mapOptions);
    if (localStorage.getItem("idRide") !== null) {
      gps.setRoute(cLatitude, cLongitude);
      var latLong = new google.maps.LatLng(aLatitude, aLongitude);
      map.setCenter(latLong);
    } else {
      marker_client.setMap();
    }
    marker_worker = new google.maps.Marker({
      position: latLong,
      map: map,
      icon: "img/marker-moto.svg"
    });
  },

  /**
   * Cria rota entre o ponto atual do mototaxista e o ponto obtido do passageiro
   * @param {float} latitude
   * @param {float} longitude
   */
  setRoute: function(latitude, longitude) {
    directionsService = new google.maps.DirectionsService();
    var rendererOptions = {
      suppressMarkers: true
        // suppressPolylines: true
    };
    directionsDisplay = new google.maps.DirectionsRenderer(rendererOptions);
    directionsDisplay.setMap(map);
    var start = new google.maps.LatLng(aLatitude, aLongitude);
    var end = new google.maps.LatLng(latitude, longitude);
    var request = {
      origin: start,
      destination: end,
      travelMode: google.maps.TravelMode.DRIVING
    };
    directionsService.route(request, function(result, status) {
      if (status == google.maps.DirectionsStatus.OK) {
        directionsDisplay.setDirections(result);
        marker_client = new google.maps.Marker({
          position: end,
          map: map,
          icon: "img/marker-client.svg"
        });
      } else {
        $.alert('Problemas com as rotas: ' + status);
      }
    });
  },

  /**
   * Permite que a localização seja reajustada para a localização inicial
   * ao clicar no botão de ajustar posição
   * @return {null}
   */
  centerLocationButton: function() {
    $(".updateGps").on("tapstart", function() {
      var latLong = new google.maps.LatLng(aLatitude, aLongitude);
      map.setCenter(latLong);
    });
  },

  /**
   * Envia ping para o servidor com informações do usuário
   * @return {null}
   */
  notifyMotion: function() {
    user.ping();
  },

  /**
   * Calcula a distância entre dois pontos
   * @param  {float} - Latitude do primeiro ponto
   * @param  {float} - Longitude do primeiro ponto
   * @param  {float} - Latitude do segundo ponto
   * @param  {float} - Longitude do segundo ponto
   * @return {float} - Distância em metros entre os dois pontos
   */
  getDistance: function(lat1, lon1, lat2, lon2) {
    var raio = 6378137; // Raio da terra em metros
    var dLatitude = gps.deg2rad(lat2 - lat1); // deg2rad
    var dLongitude = gps.deg2rad(lon2 - lon1);
    var calculo = Math.sin(dLatitude / 2) * Math.sin(dLatitude / 2) +
      Math.cos(gps.deg2rad(lat1)) * Math.cos(gps.deg2rad(lat2)) *
      Math.sin(dLongitude / 2) * Math.sin(dLongitude / 2);
    var resultado = 2 * Math.atan2(Math.sqrt(calculo), Math.sqrt(1 - calculo));
    var distancia = raio * resultado; // Distância em metros
    return distancia;
  },

  /**
   * Método para conversão de radianos
   * @param  {int}
   * @return {float} - Valor convertido
   */
  deg2rad: function(deg) {
    return deg * (Math.PI / 180);
  }

};
